//  File:        SaveLoadFile.java
//  Copyright:   Copyright (C) 2010 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Last edit:   18.06.2013
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo.helpers;

import de.alag.util.FileLogManager;
import java.io.*;
import static java.lang.System.arraycopy;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JTable;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import de.alag.vdv.ka.tlv.directory.Applikationsverzeichnis;

/**
 * Save und Load handling. Für Files.
 * @author Frank Hamm
 */
public class SaveLoadFile {
    /**
     * Der globale @see java.util.logging.Logger Logger für diese Klasse.
     */
    private static final Logger LOGGER = Logger.getLogger(SaveLoadFile.class.getName());

    /**
     * Singletoninstanz der Klasse SaveLoadFile
     */
    private static SaveLoadFile instance = null;

    /**
     * Falls noch keine Instanz der Klasse SaveLoadFile existiert, wird eine Instanz erzeugt und diese zurück gegeben.
     * Andernfalls wird die existierende Instanz zurück geliefert.
     * @return Singletoninstanz der Klasse SaveLoadFile.
     */
    public static SaveLoadFile getInstance() {
        if (instance == null) {
            instance = new SaveLoadFile();
        }
        return instance;
    }

    /**
     * Kontruktor.
     */
    private SaveLoadFile() {
        FileLogManager.getInstance().registerLogger(LOGGER);
    }

    /**
     * Lädt die Objekte (Applikationsverzeichnis, Applikationslogbuch, SeparateDatenKundendaten, SeparateDatenWes und n* SeparateDatenBerechtigung) aus der Datei
     * loadFileName und gibt die Objekte in obiger Reihenfolge in einem Object[] zurück.
     * @param loadFileName
     * @return Das Object[], das die Objekte der Klassen Applikationsverzeichnis, Applikationslogbuch, SeparateDatenKundendaten, SeparateDatenWes und n* SeparateDatenBerechtigung in dieser Reihenfolge enthält.
     */
    public Applikationsverzeichnis load(final File loadFileName) {
        FileInputStream fis = null;
        ObjectInputStream ois = null;
        LOGGER.finest("Lese Objekte von Datei...");
        try {
            fis = new FileInputStream(loadFileName);
            ois = new ObjectInputStream(fis);

            Applikationsverzeichnis app = (Applikationsverzeichnis) ois.readObject();
            LOGGER.finest("Applikationsverzeichnis gelesen");
            
            ois.close();
            fis.close();
            return app;
        } catch (Exception ex) {
            LOGGER.severe("Objekte konnten nicht geladen werden.");
            LOGGER.log(Level.FINE, null, ex);
            return null;
        }
        finally {
            if (ois != null) {
                try {
                    ois.close();
                } catch (Exception ex) {
                    LOGGER.log(Level.FINE, null, ex);
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (Exception ex) {
                    LOGGER.log(Level.FINE, null, ex);
                }
            }
        }
    }

    /**
     * Funktion zum Speichern der Objekte, die den Daten eines KundenMediums entsprechen.
     * @param saveFileName
     * @param app @see Applikationsverzeichnis
     * @return <b>true</b>: Die Objekte wurden erfolgreich gespeichert; <b>false</b>: Die Objekte konnten nicht gespeichert werden.
     */
    public boolean save(final File saveFileName,
                        final Applikationsverzeichnis app) {
        FileOutputStream fos = null;
        ObjectOutputStream oos = null;
        try {
            fos = new FileOutputStream(saveFileName);
            oos = new ObjectOutputStream(fos);

            LOGGER.finest("Schreibe Objekte in Datei...");
            oos.writeObject(app);
            LOGGER.finest("Applikationsverzeichnis geschrieben");
            
            oos.close();
            fos.close();
        } catch (Exception ex) {
            LOGGER.severe("Objekte konnten nicht gespeichert werden.");
            LOGGER.log(Level.FINE, null, ex);
            return false;
        }
        finally {
            if (oos != null) {
                try {
                    oos.close();
                } catch (Exception ex) {
                    LOGGER.log(Level.FINE, null, ex);
                }
            }
            if (fos != null) {
                try {
                    fos.close();
                } catch (Exception ex) {
                    LOGGER.log(Level.FINE, null, ex);
                }
            }
        }
        return true;
    }

    /**
     * Funktion zum Speichern eines Views
     * @param saveFileName Name der SpeicherDatei.
     * @param table Tabelle, die gespeichert werden soll.
     * @return <b>true</b>: Tabelle erfolgreich gespeichert; <b>false</b>: Tabelle konnte nicht gespeichert werden.
     */
    public boolean save(final File saveFileName, final JTable table) {
        String csvData = "";
        TableModel model = table.getModel();

        for (int i = 0; i < model.getColumnCount(); i++) {
            csvData = csvData + model.getColumnName(i) + "\t";
        }

        csvData += "\n";
        for (int y = 0; y < model.getRowCount(); y++) {
            for (int x = 0; x < model.getColumnCount(); x++) {
                int col = table.convertColumnIndexToView(x);
                String curVal = "";
                if (model.getValueAt(y, col) != null) {
                    if (model.getValueAt(y, col).toString().contains("<html>")) {
                        String buffer = model.getValueAt(y, col).toString();
                        buffer = buffer.replace("<html>", "");
                        buffer = buffer.replace("</html>", "");
                        buffer = buffer.replace("<b>", "");
                        buffer = buffer.replace("</b>", "");
                        buffer = buffer.replace("<br>", "");
                        curVal = buffer;
                    } else {
                        curVal = model.getValueAt(y, col).toString();
                    }
                }

                if (curVal == null || model.getValueAt(y, col) == null) {
                    curVal = "";
                }

                csvData = csvData + curVal + "\t";
                if (x == model.getColumnCount() - 1) {
                    csvData += "\n";
                }
            }
        }

        try {
            FileWriter writer = new FileWriter(saveFileName);
            writer.write(csvData);
            writer.flush();
            writer.close();
            writer = null;
            csvData = null;
        } catch(IOException ex) {
        }
        return true;
    }

    /**
     * Funktion zum Validieren des Dateinamen. Name + Endung.<br>
     * Wenn die Endung nicht vorhanden ist oder eine andere Endung vorhanden ist wird die durch den @see FileFilter spezifizierte Endung angehängt.<br>
     * Die eventuell bereits vorhandene Dateiendung bleibt als Teil des Dateinamens bestehen!
     * @param file Dateiname.
     * @param filter @see FileFilter der die gewünschte Endung spezifiziert.
     * @return Datei mit der zum @see FileFilter filter passenden Endung.
     */
    public File validateFileName(final File file, final FileFilter filter) {
        if (filter.accept(file)) {
            return file;
        }

        String fileName = file.getName();

        final String extension = filter.toString();
        String newFilename = fileName + "." + extension;
        File newFile = new File(file.getParent(), newFilename);
        return newFile;
    }

    /**
     * Lädt eine Datei und gibt ein Tablemodel zurück mit den geladenen Daten.
     * @param openFile Zu öffnende Datei.
     * @return TableModel mit den geladenen Daten.
     * @throws IOException Exception beim Öffenen.
     */
    public DefaultTableModel open(final File openFile) throws IOException {
        BufferedReader reader = null;
        String strLine = "";
        reader = new BufferedReader(new FileReader(openFile));
        int rows = 0;
        while ((strLine = reader.readLine()) != null) {
            rows++;
        }

        String[][] tabellenInhaltBuffer = null;
        boolean ersterDurchlauf = true;
        reader = new BufferedReader(new FileReader(openFile));
        while ((strLine = reader.readLine()) != null) {
            String[] items = strLine.split("\t");

            if (ersterDurchlauf) {
                tabellenInhaltBuffer = new String[rows][items.length];
                ersterDurchlauf = false;
                rows = 0;
            }
            arraycopy(items, 0, tabellenInhaltBuffer[rows], 0, items.length);

            rows++;
        }

        DefaultTableModel tableModel = new DefaultTableModel(tabellenInhalt(tabellenInhaltBuffer), columnName(tabellenInhaltBuffer)){
                @Override
                public Class<?> getColumnClass(int columnIndex) {
                    return String.class;
                }
            };

        return tableModel;
    }

    /**
     * Erstellt den Column Name aus geladener Datei.
     * @param tabellenInhaltBuffer Inhalt des TabellenBuffers.
     * @return
     */
    private String[] columnName(final String[][] tabellenInhaltBuffer) {
        return tabellenInhaltBuffer[0];
    }

    /**
     * Erstellt den Tabelleninhalt.
     * @param tabellenInhaltBuffer Buffer des TabellenInhaltes.
     * @return TabellenInahlt.
     */
    private String[][] tabellenInhalt(String[][] tabellenInhaltBuffer) {
        String[][] tabellenInhalt = new String[tabellenInhaltBuffer.length - 1][tabellenInhaltBuffer[0].length];
        for (int i = 0; i < tabellenInhalt.length; i++) {
            tabellenInhalt[i] = tabellenInhaltBuffer[i + 1];
        }

        return tabellenInhalt;
    }
}
