//  File:        UmbruchTableCellRenderer.java
//  Copyright:   Copyright (C) 2010 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Last edit:   18.06.2013
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo.helpers;

import de.alag.util.FileLogManager;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import javax.swing.JTable;
import java.util.logging.Logger;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;

/**
 * CellRenderer welcher Dynamischer Umbruch erlaubt.
 * @author Frank Hamm, Alexander Schuetz
 */
public class UmbruchTableCellRenderer extends JTextArea implements javax.swing.table.TableCellRenderer{

    /**
     * Der globale @see java.util.logging.Logger Logger für diese Klasse.
     */
    private static final Logger LOGGER = Logger.getLogger(UmbruchTableCellRenderer.class.getName());

    /**
     * Maximale Schriftgröße.
     */
    private final static int FONT_MAX_HEIGHT = 24;

    /**
     * Minimale Schriftgröße.
     */
    private final static int FONT_MIN_HEIGHT = 10;

    /**
     * Singletoninstanz der Klasse UmbruchTableCellRenderer
     */
    private static UmbruchTableCellRenderer instance = null;

    /**
     * Falls noch keine Instanz der Klasse UmbruchTableCellRenderer existiert, wird eine Instanz erzeugt und diese zurück gegeben.
     * Andernfalls wird die existierende Instanz zurück geliefert.
     * @return Singletoninstanz der Klasse UmbruchTableCellRenderer.
     */
    public static UmbruchTableCellRenderer getInstance() {
        if (instance == null) {
            instance = new UmbruchTableCellRenderer();
        }

        return instance;
    }

    /**
     * Standart Schriftart.
     */
    private Font font = new Font("Tahoma", 0, 11);

    /**
     * Standart Schriftart in Fett.
     */
    private Font boldFont = new Font("Tahoma", 1, 11);

    /**
     * Konstruktor
     */
    protected UmbruchTableCellRenderer() {
        setWrapStyleWord(true);
        setLineWrap(true);
        setOpaque(true);
        setBorder(new EmptyBorder(1, 2, 1, 2));
        setEditable(false);
        FileLogManager.getInstance().registerLogger(LOGGER);
    }

    /**
     * Erweiterung des TableCellRenderer.
     * @param table Die zu rendernde Tabelle.
     * @param value Der Inhalt der aktuellen Zelle.
     * @param isSelected Gibt an ob die aktuelle Zelle ausgewählt ist.
     * @param hasFocus Gibt an ob die aktuelle Zelle den Fokus hat.
     * @param row Die Zeile in der sich die aktuelle Zelle befindet.
     * @param column Die Spalte in der sich die aktuelle Zelle befindet.
     * @return JLabel, der Zelle.
     */
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            if (value != null) {
                setText(interpretColor(value.toString(), isSelected));
            } else {
                setText("");
            }

            if (table.getValueAt(row, 1) == null && column == 0 && getFont() != this.boldFont) {
                setFont(this.boldFont);
            } else if (getFont() != this.font) {
                setFont(this.font);
            }

            int rowHeight = (int) getMinimumSize().getHeight();
            if (rowHeight != table.getRowHeight(row) && column == 1) {
                table.setRowHeight(row, rowHeight);
            }

        return this;
    }

    private String interpretColor(String text, boolean selected) {
        int idx = text.indexOf("<color=");
        if (idx >= 0) {
            String tmp = text.substring(0, idx);
            text = text.substring(idx + 7);
            idx = text.indexOf("/>");
            String color = text.substring(0, idx);
            text = tmp + text.substring(idx + 2);
            if (selected) {
                setForeground(Color.decode(color));
                setBackground(Color.blue);
            } else {
                Color cc = Color.decode(color);
                int rgb = cc.getRGB();
                int ibr = ((rgb >> 20) & 0xf) + ((rgb >> 12) & 0xf) + ((rgb >> 4) & 0xf);
                if (ibr > 24) {
                    setForeground(Color.black);
                } else {
                    setForeground(Color.white);
                }
                this.setBackground(cc);
            }
        } else {
            if (selected) {
                setBackground(Color.blue);
                setForeground(Color.white);
            } else {
                setBackground(Color.white);
                setForeground(Color.black);
            }
        }
        return text;
    }
    
    /**
     * Vergrößern der Schriftart.
     */
    public void increaseFont() {
        if (this.font.getSize() < FONT_MAX_HEIGHT) {
            this.font = new Font("Tahoma", 0, this.font.getSize() + 2);
            this.boldFont = new Font("Tahoma", 1, this.boldFont.getSize() + 2);
            LOGGER.info("Schrift wurde vergrößert.");
        } else {
            LOGGER.info("Schriftgröße ist bereits maximal und wird nicht weiter vergrößert.");
        }
    }

    /**
     * Verkleinern der Schriftart.
     */
    public void decreaseFont() {
        if (this.font.getSize() > FONT_MIN_HEIGHT) {
            this.font = new Font("Tahoma", 0, this.font.getSize() - 2);
            this.boldFont = new Font("Tahoma", 1, this.boldFont.getSize() - 2);
            LOGGER.info("Schrift wurde verkleinert.");
        } else {
            LOGGER.info("Schriftgröße ist bereits minimal und wird nicht weiter verkleinert.");
        }
    }
}
